# Generated by Django 3.1.5 on 2021-02-28 17:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('id_trust', '0004_miscommunication'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Interaction',
            new_name='Dialog',
        ),
    ]

    atomic = False



"""id_trust URL Configuration
"""
from django.urls import path
from finitelycomputable.idtrust_django import views

app_name = 'id_trust'
urlpatterns = [
    path('', views.home, name='blind_home'),
    path('choose_miscommunication', views.home,
        {'blind': False}, name='reveal_home'),
    path('interact/<int:pk>', views.interact, {'blind': True},
        name='blind_interact'),
    path('interact/<int:pk>/reveal_miscommunication', views.interact,
        {'blind': False}, name='reveal_interact'),
    path('class_home', views.Home.as_view(), name='class_home'),
    path('class_interact/<int:pk>', views.Interact.as_view(),
        name='class_interact'),
    path('exchange_create/<int:interaction_id>', views.ExchangeCreate.as_view(),
        name='exchange_create'),
]



# Generated by Django 3.1 on 2020-08-26 00:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('id_trust', '0002_auto_20180311_0154'),
    ]

    operations = [
        migrations.AddField(
            model_name='interaction',
            name='user_guess',
            field=models.CharField(blank=True, choices=[('C', 'copy cat'), ('X', 'cheat'), ('I', 'innocent'), ('G', 'grudger'), ('D', 'detective')], max_length=1),
        ),
    ]



from django.test import Client, TestCase

from . import models


def default_create():
    models.Dialog.objects.create(
        foil_strategy='C',
        user_miscommunication=0.0,
        foil_miscommunication=0.0,
    )


class IdTrustViews(TestCase):
    def setUp(self):
        self.c = Client()

    def test_get_blind_home_200(self):
        resp = self.c.get('/identification_of_trust/')
        self.assertEqual(resp.status_code, 200)

    def test_get_reveal_home_200(self):
        resp = self.c.get('/identification_of_trust/choose_miscommunication')
        self.assertEqual(resp.status_code, 200)

    def test_get_interact_404(self):
        resp = self.c.get('/identification_of_trust/interact/1')
        self.assertEqual(resp.status_code, 404)

    def test_get_interact_200(self):
        default_create()
        resp = self.c.get('/identification_of_trust/interact/1')
        self.assertEqual(resp.status_code, 200)

    def test_post_blind_home_creates_interaction(self):
        self.assertEqual(models.Dialog.objects.count(), 0)
        resp = self.c.post('/identification_of_trust/',
                {'user_intent': 'Trust'})
        self.assertEqual(resp.status_code, 302)
        self.assertEqual(models.Dialog.objects.count(), 1)

    def test_post_reveal_home_creates_interaction(self):
        self.assertEqual(models.Dialog.objects.count(), 0)
        resp = self.c.post('/identification_of_trust/', {
            'user_intent': 'Trust',
            'user_miscommunication': 0.1,
            'foil_miscommunication': 0.1,
        })
        self.assertEqual(resp.status_code, 302)
        self.assertEqual(models.Dialog.objects.count(), 1)

    def test_post_trust_creates_exchange(self):
        self.assertEqual(models.Exchange.objects.count(), 0)
        default_create()
        resp = self.c.post('/identification_of_trust/interact/1',
                {'user_intent': 'Trust'})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(models.Exchange.objects.count(), 1)

    def test_post_distrust_creates_exchange(self):
        self.assertEqual(models.Exchange.objects.count(), 0)
        default_create()
        resp = self.c.post('/identification_of_trust/interact/1',
                {'user_intent': 'Distrust'})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(models.Exchange.objects.count(), 1)

    def test_post_user_guess_sets_user_guess(self):
        self.assertEqual(models.Exchange.objects.count(), 0)
        default_create()
        resp = self.c.post('/identification_of_trust/interact/1',
                {'user_guess': 'innocent'})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(models.Exchange.objects.count(), 0)
        self.assertEqual(models.Dialog.objects.first().user_guess,
                'innocent')

    def test_class_get_home_200(self):
        resp = self.c.get('/identification_of_trust/class_home')
        self.assertEqual(resp.status_code, 200)



# Generated by Django 2.0.2 on 2018-02-13 03:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Exchange',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user_trust', models.BooleanField()),
                ('foil_trust', models.BooleanField()),
            ],
        ),
        migrations.CreateModel(
            name='Interaction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start_time', models.DateTimeField()),
                ('foil_strategy', models.CharField(max_length=1)),
            ],
        ),
        migrations.AddField(
            model_name='exchange',
            name='interaction',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='id_trust.Interaction'),
        ),
    ]



version = '21.2'
#version += 'a1'

base_setup = {
    'package_dir': {'': '.'},
    'package_data': {'': ['templates/*/*']},
    'data_files': [('', ['README.rst', 'LICENSE.txt'])],
    'include_package_data': False,
    'license': 'AGPL-3.0-only',
    'python_requires': '>=3.6',
    'author': 'Samuel Newbold',
    'author_email': 'sam@rwsh.org',
    'classifiers': [
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
    ],
    'project_urls': {
        'Documentation': 'https://github.com/thrasymache/microsites',
        'Source': 'https://github.com/thrasymache/microsites',
    },
}

django_classifiers = [
        'Framework :: Django',
        'Framework :: Django :: 3.0',
    ]

django_setup = base_setup.copy()
django_setup['classifiers'] = django_setup['classifiers'].copy()
django_setup['classifiers'].extend(django_classifiers)

django_version = 'Django~=3.1'
flask_version = 'Flask~=1.1'
cherrypy_version = 'CherryPy~=18.6'
morepath_version = 'morepath~=0.19'
falcon_version = 'falcon~=2.0'

wsgi_extras_require={
        'bjoern': ['bjoern~=3.0'],
        'cheroot': ['cheroot~=8.0'],
        'cherrypy': ['cherrypy~=18.0'],
        'gunicorn': ['gunicorn~=20.0'],
        'waitress': ['waitress~=1.4'],
        }



# Generated by Django 2.0.2 on 2018-03-11 01:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('id_trust', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='interaction',
            name='foil_strategy',
            field=models.CharField(choices=[('C', 'copy cat'), ('X', 'cheat'), ('I', 'innocent'), ('G', 'grudger'), ('D', 'detective')], max_length=1),
        ),
        migrations.AlterField(
            model_name='interaction',
            name='start_time',
            field=models.DateTimeField(auto_now_add=True),
        ),
    ]



#! /usr/bin/env python
import os
from setuptools import find_namespace_packages, setup
from finitelycomputable_microsites_setup import (
        version, django_setup, wsgi_extras_require, django_version,
)

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='finitelycomputable-idtrust-django',
    version=version,
    packages=find_namespace_packages(),
    entry_points={
        'console_scripts': [
            'finitelycomputable-idtrust-django = finitelycomputable.idtrust_django:run']
        },
    description='A Django microsite to explore identifying game-theory strategies',
    long_description=README,
    long_description_content_type="text/x-rst",
    scripts=['finitelycomputable_microsites_setup.py'],
    install_requires=[django_version],
    extras_require=wsgi_extras_require,
    url='http://www.finitelycomputable.net/identification_of_trust',
    **django_setup
)



from django.db import models
from django.urls import reverse


def alternate_strategy(inputs, strategy):
    result = []
    for n in range(len(inputs)+1):
        subset = inputs[:n]
        result.append(strategy(subset))
    return result


def pct_deviation(outputs, inputs, strategy):
    return round(100 * deviation(outputs, inputs, strategy))

def deviation(outputs, inputs, strategy):
    result = 0
    for n in range(len(inputs)):
        subset = inputs[:n]
        result += outputs[n] != strategy(subset)
    return result / len(inputs)


class Strategy(models.TextChoices):
    COPY_CAT = "C", 'copy cat'
    CHEAT = "X", 'cheat'
    INNOCENT = "I", 'innocent'
    GRUDGER = "G", 'grudger'
    DETECTIVE = "D", 'detective'

    @classmethod
    def impl(cls, enum_val):
        if enum_val == Strategy.COPY_CAT:
            return cls.copy_cat
        elif enum_val == Strategy.CHEAT:
            return cls.cheat
        elif enum_val == Strategy.INNOCENT:
            return cls.innocent
        elif enum_val == Strategy.GRUDGER:
            return cls.grudger
        elif enum_val == Strategy.DETECTIVE:
            return cls.detective
        else:
            raise KeyError(f'Unrecognized enum_val: {enum_val}')

    @classmethod
    def copy_cat(cls, exchanges):
        return exchanges[-1] if len(exchanges) else True

    @classmethod
    def cheat(cls, exchanges):
        return False

    @classmethod
    def innocent(cls, exchanges):
        return True

    @classmethod
    def grudger(cls, exchanges):
        return not [i for i in exchanges if not i]

    @classmethod
    def detective(cls, exchanges):
        if len(exchanges) in [0, 2, 3]:
            return True
        elif len(exchanges) in [1]:
            return False
        elif [i for i in exchanges if not i]:
            return cls.copy_cat(exchanges)
        else:
            return cls.cheat(exchanges)


# TODO add exchange time to the exchange models, make sure that it's
# sub-second, or have a better strategy


class Dialog(models.Model):
    start_time = models.DateTimeField(auto_now_add=True)
    foil_strategy = models.CharField(choices=Strategy.choices, max_length=1)
    user_miscommunication  = models.FloatField()
    foil_miscommunication  = models.FloatField()
    user_guess = models.CharField(choices=Strategy.choices, max_length=1,
            blank=True)


    def score(self):
        user_result = foil_result = 0
        for e in self.exchange_set.all():
            if e.user_intent and e.foil_effect:
                user_result += 2
                foil_result += 2
            elif e.user_intent:
                user_result -= 1
                foil_result += 3
            elif e.foil_effect:
                user_result += 3
                foil_result -= 1
            else:
                pass
        return user_result, foil_result

    def get_absolute_url(self):
        return reverse('id_trust:blind_interact', kwargs={'pk': self.pk})


class Exchange(models.Model):
    interaction = models.ForeignKey(Dialog, on_delete=models.CASCADE)
    user_intent = models.BooleanField()
    user_effect = models.BooleanField()
    foil_intent = models.BooleanField()
    foil_effect = models.BooleanField()
